//parameter GPIO_NUM_INPUTS = 4;

`include "config.sv"

module gpio_decoder(clk, reset, inputs, gpio_rising, gpio_falling);

	input  logic clk, reset;
	input  logic [GPIO_NUM_INPUTS-1:0] inputs      ;
	output logic [GPIO_NUM_INPUTS-1:0] gpio_rising ;
	output logic [GPIO_NUM_INPUTS-1:0] gpio_falling;
	
	genvar i;
	generate
	
		for (i = 0; i < GPIO_NUM_INPUTS; i++) begin: gpio_decoder_gen
			
			// Store hysterisis of input
			logic last_state_reg, current_state_reg;
			
			// Update current and last state
			always @(posedge clk) begin
				if (reset) begin
					last_state_reg    <= 0;
					current_state_reg <= 0;
				end else begin
					last_state_reg    <= current_state_reg;
					current_state_reg <= inputs[i];
				end
			end
			
			assign gpio_rising[i]  = ~last_state_reg &  current_state_reg;
			assign gpio_falling[i] = last_state_reg  & ~current_state_reg;
			
		end
	
	endgenerate


endmodule