// =================================================
// ring_buffer_manager
// =================================================

module ring_buffer_manager(
	ring_buffer_size,
	current_write_pointer, next_write_pointer,
	current_read_pointer, next_read_pointer,
	can_read, can_write
);


input  logic [31:0] ring_buffer_size;
input  logic [31:0] current_write_pointer;
input  logic [31:0] current_read_pointer;

output logic [31:0] next_write_pointer;
output logic [31:0] next_read_pointer;

output logic can_read;
output logic can_write;

// Compute next pointers
always_comb begin
	// Write
	if (current_write_pointer == ring_buffer_size - 1) begin
		next_write_pointer = 32'd0;
	end else begin
		next_write_pointer = current_write_pointer + 32'd1;
	end
	
	// read
	if (current_read_pointer == ring_buffer_size - 1) begin
		next_read_pointer = 32'd0;
	end else begin
		next_read_pointer = current_read_pointer + 32'd1;
	end
end

// Ring buffer is full if read_ptr = write_ptr - 1 mod ring_buffer_size
always_comb begin
	
	// Check if empty
	can_read = next_read_pointer != current_write_pointer;
	
	// Check if full
	can_write = next_write_pointer != current_read_pointer;
	
	
end


endmodule