`timescale 1ns/1ns

`include "config.sv"

module testbench();
	
	logic clk_100, reset;
	
	logic [31:0] index;
	
	logic sda, scl;
	
	logic miso, mosi, sck, cs;
	
	logic toggle;

	//Initialize clocks and index
	initial begin
		clk_100 = 1'b0;
		index  = 32'd0;
		toggle = 1'b0;
		//testbench_out = 15'd0 ;
	end
	
	//Toggle the clocks
	always begin
		#5
		clk_100  = !clk_100;
	end
	//toggle spi clock 
	always begin 
		#1
		if (cs == 1'b0)begin 
			sck = !sck;
		end 
	end 
	
	always begin
		#20
		toggle = !toggle;
	end
	
	//Intialize and drive signals
	integer i;
	initial begin
		reset  = 1'b0;
		sda = 1;
		scl = 1;
		
		cs = 1;
		miso = 0;
		mosi = 0;
		sck = 0;
		
		#10
		reset  = 1'b1;
		#100
		reset  = 1'b0;
		#10
		
		// Test I2C Transaction 
		
		// Start condition
		sda = 0;
		#10
		scl = 0;
		#10
		
		// Data bytes
		sda = 1; // write all ones
		for (i = 0; i < 8; i++) begin
			scl = 0;
			#10
			
			scl = 1; // rising click edge
			#10
			
			sda = 1;
		end
		
		// Ack / Nack
		scl = 0;
		sda = 0; // ack
		#10
		scl = 1;
		#10
		
		// Stop condition
		scl = 1;
		#10
		sda = 0;
		#10
		sda = 1;
		
		
		//test spi_decoder 
		// Start condition	
		// Data bytes
		cs = 0; // write all ones
		for (i = 0; i < 8; i++) begin
			 #2
          mosi = !mosi;
			 miso = 0;
	   end
		for (i = 0; i < 8; i++) begin
			 #2
          mosi = 0;
			 miso = !miso;
	   end
		cs = 1;
		for (i = 0; i < 8; i++) begin
			 #2
          mosi = !mosi;
			 miso = 0;
	   end
		for (i = 0; i < 8; i++) begin
			 #2
          mosi = 0;
			 miso = !miso;
	   end
	end
	
	//Increment index
	always @ (posedge clk_100) begin
		index  <= index + 32'd1;
	end
	
	logic [3:0] reqs, grants;
	assign reqs = 4'b1111;
	
	logic [3:0] inputs;
	assign inputs[0] = sda;
	assign inputs[1] = scl;
	assign inputs[2] = toggle;
	assign inputs[3] = 0;
	
	//i2c_decoder decoder(clk_100, reset, sda, scl);
	//spi_decoder decoder2(clk_100, reset, sck, miso, mosi, cs);
	//vc_RoundRobinArb roundrobin(clk_100, reset, reqs, grants);
	
	
	logic [RINGBUF_ADDR_SIZE-1:0] data_read_addr;
	logic [RINGBUF_ADDR_SIZE-1:0] data_read_next_addr;
	logic [RINGBUF_WORD_SIZE-1:0] data_read_data;
	logic                         data_can_read;
	
	logic [DPR_ADDR_SIZE-1:0]     sample_write_addr;
	logic [DPR_ADDR_SIZE-1:0]     sample_write_addr_next;
	logic                         sample_can_write;

	
	always @(posedge clk_100) begin
		if (reset) begin
			data_read_addr <= 0;
			sample_write_addr <= 1;
		end else begin
			if (data_can_read && sample_can_write) begin
				data_read_addr <= data_read_next_addr;
				sample_write_addr <= sample_write_addr_next;
			end
		end
	end
	
	ring_buffer_manager hps_buf_manager(
		.ring_buffer_size				(DPR_NUM_ENTRIES),
		
		.current_write_pointer		(sample_write_addr), 
		.next_write_pointer			(sample_write_addr_next),
		.can_write						(sample_can_write), 

		.current_read_pointer		(0), 
		.next_read_pointer			(),
		.can_read						()
	);

	
	dual_port_ram dpr(
		.q				(), 
		.d				(data_read_data), 
		.wr_addr		(sample_write_addr), 
		.rd_addr		(), 
		.we			(data_can_read), 
		.clk1			(clk_100), 
		.clk2			(clk_100)
	);

	analyzer_block ablock(
		.clk				(clk_100), 
		.reset			(reset), 
		.inputs			(inputs), 
		.configuration	(2'd1), // GPIO 
		.trig_config	(2'd1), // GPIO
		.trig_cond		(2'd00010001), // rising or falling edge of sck
		
		.sampler_id    (2'd0),
		
		// Data output port
		.data_read_data			(data_read_data), 
		.data_read_addr			(data_read_addr), 
		.data_read_next_addr		(data_read_next_addr), 
		.data_can_read				(data_can_read)
	);
	
endmodule
